// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus, Florian Fischer
#pragma once

#include <functional>
#include <sstream>	// IWYU pragma: keep

#include "emper-config.h"	 // IWYU pragma: keep

using func_t = std::function<void()>;

#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)
#define DIE die(__FILE__ ":" TOSTRING(__LINE__), false)

// See Debug.hpp why we disable clang-format for the multi statement macro definitions
// clang-format off

// NOLINTNEXTLINE(bugprone-macro-parentheses)
#define DIE_MSG(x) do { std::stringstream sst; sst << __FILE__ ":" TOSTRING(__LINE__) " " << x; die(sst.str().c_str(), false); } while (false)
// NOLINTNEXTLINE(bugprone-macro-parentheses)
#define DIE_MSG_ERRNO(x) do { std::stringstream sst; sst << __FILE__ ":" TOSTRING(__LINE__) " " << x; die(sst.str().c_str(), true); } while (false)

// clang-format on

// We compile with -fno-exceptions for the moment.
//#define THROW(x) do { throw std::runtime_error(x); } while (false)

#define likely(x) __builtin_expect(!!(x), 1)
#define unlikely(x) __builtin_expect(!!(x), 0)

#define ALIGN_TO_CACHE_LINE alignas(64)

[[noreturn]] void die(const char* message, bool usePerror);

using WORD = unsigned int;

#ifdef EMPER_STATS
#include <chrono>	 // IWYU pragma: keep
#define TIME_NS(code, record_func)                                                         \
	auto _time_ns_start = std::chrono::high_resolution_clock::now();                         \
	code;                                                                                    \
	auto _time_ns_end = std::chrono::high_resolution_clock::now();                           \
	auto _time_ns_duration =                                                                 \
			std::chrono::duration_cast<std::chrono::nanoseconds>(_time_ns_end - _time_ns_start); \
	record_func(_time_ns_duration);

#else
#define TIME_NS(code, record) code
#endif
