// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#include "Emper.hpp"

#include <cerrno>
#include <cstdint>
#include <stdexcept>

#include "Common.hpp"
#include "Runtime.hpp"
#include "emper-common.h"
#include "emper-version.h"
#include "io/Future.hpp"

namespace emper {

auto getFullVersion() -> std::string { return EMPER_FULL_VERSION; }

static void ensure_no_current_runtime() {
	Runtime* runtime = Runtime::getRuntime();
	if (runtime) {
		throw std::runtime_error("Runtime already initialized");
	}
}

void init_runtime() {
	ensure_no_current_runtime();

	// NOLINTNEXTLINE(clang-analyzer-cplusplus.NewDeleteLeaks)
	new Runtime();
	// NOLINTNEXTLINE(clang-analyzer-cplusplus.NewDeleteLeaks)
}

void init_runtime(workerid_t worker_count) {
	ensure_no_current_runtime();

	// NOLINTNEXTLINE(clang-analyzer-cplusplus.NewDeleteLeaks)
	new Runtime(worker_count);
	// NOLINTNEXTLINE(clang-analyzer-cplusplus.NewDeleteLeaks)
}

void destroy_runtime() {
	Runtime* runtime = Runtime::getRuntime();
	if (!runtime) {
		throw std::runtime_error("No runtime to destroy");
	}

	delete runtime;
}

auto sleep(unsigned int seconds) -> bool {
	if constexpr (!emper::IO) {
		DIE_MSG("sleep requires emper::io");
	}

	emper::io::AlarmFuture::Timespec ts = {.tv_sec = seconds, .tv_nsec = 0};
	emper::io::AlarmFuture alarm(ts);
	int32_t res = alarm.submitAndWait();

	return res == -ETIME;
}

}	 // namespace emper
