// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus
#pragma once

#include <functional>

#include "Runtime.hpp"

template <typename D>
class WorkerLocalData {
 private:
	std::function<D*(void)> getWorkerLocalPtrCallback;
	std::vector<D*> workerLocalData;

 public:
	WorkerLocalData(std::function<D*(void)> getWorkerLocalPtrCallback, Runtime& runtime)
			: getWorkerLocalPtrCallback(getWorkerLocalPtrCallback) {
		auto workerCount = runtime.getWorkerCount();
		workerLocalData = std::vector<D*>(workerCount);
		runtime.addNewWorkerHook([this, getWorkerLocalPtrCallback](workerid_t workerId) {
			D* workerLocalDataPtr = getWorkerLocalPtrCallback();
			this->workerLocalData.at(workerId) = workerLocalDataPtr;
		});
	}

	auto getSnapshot() -> std::vector<D> {
		std::vector<D> res;
		res.reserve(workerLocalData.size());
		for (D* data : workerLocalData) {
			res.emplace_back(*data);
		}
		return res;
	}

	void forEach(std::function<void(D*)> callback) {
		for (D* data : workerLocalData) {
			callback(data);
		}
	}
};
