// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#pragma once

#include <cassert>
#include <functional>
#include <utility>

#include "Common.hpp"
#include "Fiber.hpp"
#include "Runtime.hpp"
#include "SynchronizedFiber.hpp"

void async(Fiber* fiber) {
	assert(fiber != nullptr);
	Runtime* runtime = Runtime::getRuntime();
	runtime->schedule(*fiber);
}

void async(Fiber::fiber_fun_t function, void* arg) {
	Fiber* fiber = Fiber::from(std::move(function), arg);
	async(fiber);
}

void async(const Fiber::fiber_fun0_t& function) {
	Fiber* fiber = Fiber::from(function);
	async(fiber);
}

void async(Fiber::fiber_fun_t function, void* arg, workeraffinity_t* affinity) {
	Fiber* fiber = Fiber::from(std::move(function), arg, affinity);
	async(fiber);
}

void async(const Fiber::fiber_fun0_t& function, workeraffinity_t* affinity) {
	Fiber* fiber = Fiber::from(function, affinity);
	async(fiber);
}

template <typename S>
void spawn(Fiber::fiber_fun_t function, void* arg, S& semaphore) {
	Fiber* fiber = SynchronizedFiber::from(function, arg, semaphore);
	async(fiber);
}

template <typename S>
void spawn(Fiber::fiber_fun0_t function, S& semaphore) {
	Fiber* fiber = SynchronizedFiber::from(function, semaphore);
	async(fiber);
}

template <typename S>
void spawn(Fiber::fiber_fun_t function, void* arg, workeraffinity_t* affinity, S& semaphore) {
	Fiber* fiber = SynchronizedFiber::from(function, arg, affinity, semaphore);
	async(fiber);
}

template <typename S>
void spawn(Fiber::fiber_fun0_t function, workeraffinity_t* affinity, S& semaphore) {
	Fiber* fiber = SynchronizedFiber::from(function, affinity, semaphore);
	async(fiber);
}

namespace emper {

void init_runtime();

void init_runtime(workerid_t worker_count);

void destroy_runtime();

void yield() {
	Runtime* runtime = Runtime::getRuntime();
	runtime->yield();
}

auto sleep(unsigned int seconds) -> bool;

}	 // namespace emper
