// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Fischer
#pragma once

#include <pthread.h>	// for pthread_t

#include <cstdint>
#include <mutex>
#include <string>

#include "emper-common.h"
#include "io/IoContext.hpp"
class Runtime;

namespace emper::io {
class GlobalIoContext : public IoContext {
	friend Runtime;
	friend void ::emper_log(const std::string& prefix, const std::string& message);
	friend class Future;
	friend class SendFuture;
	friend class RecvFuture;

 private:
	GlobalIoContext(Runtime& runtime, workerid_t workerCount) : IoContext(runtime, workerCount) {
		worker = nullptr;
	}

	// This mutex is only used to protect the SQ during start up when all workers
	// register their IoContext's eventfds in parallel
	std::mutex registerLock;

	enum class PointerTags : uint16_t { Future, IoContext, TerminationEvent };

	// pthread used to monitor the CQs from worker io_urings
	// as well as handling IO requests submitted from anywhere
	pthread_t globalCompleter;

	/* function executed by the global completer thread */
	static auto globalCompleterFunc(void* arg) -> void*;

	// start the global completer thread
	void startGlobalCompleter();

	void initiateTermination();
	void waitUntilFinished() const;

	void registerWorkerIo(IoContext& workerIo);
	void prepareWorkerNotification(IoContext& workerIo);
};
}	 // namespace emper::io
