// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Fischer, 2021 Florian Schmaus
#pragma once

#include <iterator>
#include <mutex>
#include <set>

namespace emper::lib::adt {

template <class Key, class Compare = std::less<Key>, class Allocator = std::allocator<Key>>
class LockedSet {
 public:
	using set_type = std::set<Key, Compare, Allocator>;

 private:
	std::mutex _mutex;
	set_type _set;

 public:
	auto insertAndGetSize(const Key& item) -> typename set_type::size_type {
		std::lock_guard<std::mutex> lock(_mutex);
		_set.insert(item);
		return _set.size();
	}

	auto insert(const Key& item)
			-> std::pair<typename std::set<Key, Compare, Allocator>::iterator, bool> {
		std::lock_guard<std::mutex> lock(_mutex);
		return _set.insert(item);
	}

	template <class InputIt>
	void insert(InputIt first, InputIt last) {
		std::lock_guard<std::mutex> lock(_mutex);
		_set.insert(first, last);
	}

	void insert(const Key* items, unsigned count) {
		std::lock_guard<std::mutex> lock(_mutex);
		for (unsigned i = 0; i < count; ++i) {
			_set.insert(items[i]);
		}
	}

	auto erase(const Key& key) -> size_t {
		std::lock_guard<std::mutex> lock(_mutex);
		return _set.erase(key);
	}
};
}	 // namespace emper::lib::adt
