// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <atomic>

namespace adt {

template <typename T>
class MpscQueue {
 private:
	T* head;
	std::atomic<T*> tail;
	T dummy;

 public:
	MpscQueue() {
		head = &dummy;
		tail = head;
	}

	void enqueue(T* item) {
		item->setMpscNext(nullptr);
		T* prev = tail.exchange(item);
		prev->setMpscNext(item);
	}

	auto dequeue() -> T* {
		T* next = head->getMpscNext();
		if (next == nullptr) {
			// The queue is empty.
			return nullptr;
		}
		T* item = head;
		// We are possible removing  'item', set head to next item after
		// it.
		head = next;
		// But first check if item is the dummy item.
		if (item == &dummy) {
			enqueue(item);
			if (head->getMpscNext() == nullptr) {
				// The queue is empty.
				return nullptr;
			}
			head = head->getMpscNext();
			return next;
		}
		return item;
	}
};

}	 // namespace adt
