// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus, Florian Fischer
#pragma once

#include <queue>
#include <shared_mutex>

namespace lib::adt {

template <typename I>
class SharedMutexUnboundedQueue {
 private:
	std::shared_mutex queue_mutex;

	std::queue<I*> queue;

 public:
	void enqueue(I* item) {
		std::unique_lock lock(queue_mutex);
		queue.push(item);
	}

	template <class InputIt>
	void insert(InputIt begin, InputIt end) {
		std::unique_lock lock(queue_mutex);
		for (; begin != end; ++begin) {
			queue.push(*begin);
		}
	}

	void insert(I** items, unsigned count) {
		std::unique_lock lock(queue_mutex);
		for (unsigned i = 0; i < count; ++i) {
			queue.push(items[i]);
		}
	}

	auto dequeue() -> I* {
		{
			std::shared_lock lock(queue_mutex);
			if (queue.empty()) {
				return nullptr;
			}
		}

		std::unique_lock lock(queue_mutex);
		if (queue.empty()) {
			return nullptr;
		}
		I* res = queue.front();
		queue.pop();
		return res;
	}

	template <class OutputIt>
	auto dequeue(OutputIt begin, OutputIt end) -> size_t {
		{
			std::shared_lock lock(queue_mutex);
			if (queue.empty()) return 0;
		}

		OutputIt cur = begin;
		{
			std::unique_lock lock(queue_mutex);
			for (; !queue.empty() && cur != end; ++cur) {
				*cur = queue.front();
				queue.pop();
			}
		}
		return cur - begin;
	}
};
}	 // namespace lib::adt
