// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <condition_variable>
#include <mutex>

#include "Common.hpp"
namespace emper::lib::sync {

class LockedSemaphore {
 public:
	using CounterType = unsigned int;

 private:
	std::mutex m;
	std::condition_variable c;
	CounterType counter;

 public:
	LockedSemaphore(CounterType counter = 0) : counter(counter) {}

	[[nodiscard]] inline auto getValue() const -> CounterType { return counter; }

	inline void notify() {
		std::unique_lock<std::mutex> lock(m);
		++counter;
		c.notify_one();
	}

	inline void notify_many(CounterType count) {
		std::unique_lock<std::mutex> lock(m);
		counter += count;
		c.notify_all();
	}

	inline void wait() {
		std::unique_lock<std::mutex> lock(m);
		c.wait(lock, [this]() { return counter > 0; });
		counter--;
	}
};

}	 // namespace emper::lib::sync
