// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Fischer
#pragma once

#include <semaphore.h>

#include "Common.hpp"

namespace emper::lib::sync {

class PosixSemaphore {
 public:
	using CounterType = int;

 private:
	sem_t s;

 public:
	PosixSemaphore(CounterType counter = 0) {
		int err = sem_init(&s, 0, counter);
		if (unlikely(err)) {
			DIE_MSG_ERRNO("sem_init failed");
		}
	}

	~PosixSemaphore() {
		int err = sem_destroy(&s);
		if (unlikely(err)) {
			DIE_MSG_ERRNO("sem_destroy failed");
		}
	}

	[[nodiscard]] inline auto getValue() -> CounterType {
		CounterType sval;
		sem_getvalue(&s, &sval);
		return sval;
	}

	inline void notify() { sem_post(&s); }

	inline void notify_many(CounterType count) {
		for (; count > 0; --count) {
			notify();
		}
	}

	inline void wait() { sem_wait(&s); }
};

}	 // namespace emper::lib::sync
