// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Fischer
#pragma once

#include <atomic>

#include "Common.hpp"

namespace emper::lib::sync {

class SpinLock {
 private:
	std::atomic<bool> locked;

 public:
	SpinLock(bool locked = false) : locked(locked) {}

	inline void lock() {
		bool isLocked;
		do {
			isLocked = false;
		} while (!locked.compare_exchange_weak(isLocked, true, std::memory_order_acquire,
																					 std::memory_order_relaxed));
	}

	inline void unlock() { locked.store(false, std::memory_order_release); }
};
}	 // namespace emper::lib::sync
