// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Fischer
#pragma once

#include <stdexcept>

#include "CallerEnvironment.hpp"
#include "Emper.hpp"

namespace emper::sleep_strategy {

static constexpr bool needsNotifySpecific =
		(emper::IO_COMPLETER_BEHAVIOR == emper::IoCompleterBehavior::wakeup);

template <class T>
class AbstractWorkerSleepStrategy {
	[[nodiscard]] inline auto getSleeping() const -> long {
		return static_cast<T*>(this)->getSleeping();
	}

	template <CallerEnvironment callerEnvironment>
	inline void notifyMany(unsigned count) {
		static_cast<T*>(this)->template notifyMany<callerEnvironment>(count);
	}

	template <CallerEnvironment callerEnvironment>
	inline void notifyOne() {
		static_cast<T*>(this)->template notifyOne<callerEnvironment>();
	}

	template <CallerEnvironment callerEnvironment>
	inline void notifyAll() {
		static_cast<T*>(this)->template notifyAll<callerEnvironment>();
	}

	template <CallerEnvironment callerEnvironment>
	void notifySpecific(workerid_t workerId) {
		if constexpr (needsNotifySpecific) {
			static_cast<T*>(this)->template notifySpecific<callerEnvironment>(workerId);
		} else {
			throw std::logic_error(
					"Called SemaphoreWorkerSleepStrategy::notifySpecific but needsNotifySpecific is false");
		}
	}

	void sleep() { static_cast<T*>(this)->sleep(); }

	inline void printStats() { static_cast<T*>(this)->printStats(); }
};
}	 // namespace emper::sleep_strategy
