// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus
#include "FromAnywhere.hpp"

#include <memory>
#include <string>

using namespace emper::stats;

FromAnywhere::FromAnywhere(workerid_t workerCount)
		: unblockAffinitiesGeneric(std::vector<std::atomic<uint32_t>>(workerCount)),
			unblockAffinitiesIo(std::vector<std::atomic<uint32_t>>(workerCount)) {}

void FromAnywhere::recordUnblockedContext(workeraffinity_t workerAffinity,
																					BlockablePurpose blockablePurpose) {
	switch (blockablePurpose) {
		case BlockablePurpose::GENERIC:
			unblockAffinitiesGeneric[workerAffinity]++;
			break;
		case BlockablePurpose::IO:
			unblockAffinitiesIo[workerAffinity]++;
			break;
	}
}

auto FromAnywhere::operator+=(const FromAnywhere& other) -> FromAnywhere& {
	for (decltype(unblockAffinitiesGeneric)::size_type i = 0; i < unblockAffinitiesGeneric.size();
			 i++) {
		unblockAffinitiesGeneric[i] += other.unblockAffinitiesGeneric[i];
		unblockAffinitiesIo[i] += other.unblockAffinitiesIo[i];
	}
	return *this;
}

void FromAnywhere::print(std::ostream& out) {
	for (decltype(unblockAffinitiesGeneric)::size_type i = 0; i < unblockAffinitiesGeneric.size();
			 ++i) {
		out << "unblock-affinities-fa-generic" << std::to_string(i) << ": "
				<< std::to_string(unblockAffinitiesGeneric[i]) << std::endl;
	}
	for (decltype(unblockAffinitiesIo)::size_type i = 0; i < unblockAffinitiesIo.size(); ++i) {
		out << "unblock-affinities-fa-io" << std::to_string(i) << ": "
				<< std::to_string(unblockAffinitiesIo[i]) << std::endl;
	}
}
