// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus
#include "Worker.hpp"

#include <algorithm>
#include <string>

using namespace emper::stats;

thread_local Worker* emper::stats::worker = nullptr;

Worker::Worker(workerid_t workerCount)
		: unblockAffinitiesGeneric(std::vector<uint32_t>(workerCount)),
			unblockAffinitiesIo(std::vector<uint32_t>(workerCount)) {}

void Worker::recordBlockedContext(size_t blockedContextCount, BlockablePurpose blockablePurpose) {
	maxBlockedContexts = std::max(maxBlockedContexts, blockedContextCount);

	switch (blockablePurpose) {
		case BlockablePurpose::GENERIC:
			blockedGeneric++;
			break;
		case BlockablePurpose::IO:
			blockedIo++;
			break;
	}
}

void Worker::recordUnblockedContext(workeraffinity_t workerAffinity,
																		BlockablePurpose blockablePurpose) {
	switch (blockablePurpose) {
		case BlockablePurpose::GENERIC:
			unblockAffinitiesGeneric[workerAffinity]++;
			break;
		case BlockablePurpose::IO:
			unblockAffinitiesIo[workerAffinity]++;
			break;
	}
}

auto Worker::operator+=(const Worker& other) -> Worker& {
	maxBlockedContexts = std::max(maxBlockedContexts, other.maxBlockedContexts);
	blockedGeneric += other.blockedGeneric;
	blockedIo += other.blockedIo;

	for (decltype(unblockAffinitiesGeneric)::size_type i = 0; i < unblockAffinitiesGeneric.size();
			 i++) {
		unblockAffinitiesGeneric[i] += other.unblockAffinitiesGeneric[i];
		unblockAffinitiesIo[i] += other.unblockAffinitiesIo[i];
	}
	return *this;
}

void Worker::print(std::ostream& out) {
	out << "max-blocked-contexts: " << std::to_string(maxBlockedContexts) << std::endl
			<< "blocked-generic: " << std::to_string(blockedGeneric) << std::endl;
	for (decltype(unblockAffinitiesGeneric)::size_type i = 0; i < unblockAffinitiesGeneric.size();
			 ++i) {
		out << "unblock-affinities-generic" << std::to_string(i) << ": "
				<< std::to_string(unblockAffinitiesGeneric[i]) << std::endl;
	}
	out << "blocked-io: " << std::to_string(blockedIo) << std::endl;
	for (decltype(unblockAffinitiesIo)::size_type i = 0; i < unblockAffinitiesIo.size(); ++i) {
		out << "unblock-affinities-io" << std::to_string(i) << ": "
				<< std::to_string(unblockAffinitiesIo[i]) << std::endl;
	}
}
