// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus
#pragma once

#include <cstddef>	// for size_t
#include <cstdint>
#include <optional>
#include <utility>

#include "NextFiberResult.hpp"
#include "Scheduler.hpp"
#include "emper-common.h"

#ifdef EMPER_LOCKED_WS_QUEUE
#include "lib/adt/LockedQueue.hpp"
#else
#include "lib/adt/WsClQueue.hpp"
#endif

class Fiber;
class Runtime;
class RuntimeStrategy;

class AbstractWorkStealingScheduler : public Scheduler {
	template <size_t SIZE>
#ifdef EMPER_LOCKED_WS_QUEUE
	using WsQueue = adt::LockedQueue<Fiber*, SIZE>;
#else
	using WsQueue = adt::WsClQueue<Fiber*, SIZE>;
#endif

 public:
	static const int QUEUE_SIZE = 1024;

	enum class FiberSource : uintptr_t {
		local,
		hintWsq,
		hintAq,
		stolen,
		anywhereQueue,
	};

 private:
	auto nextFiberViaAnywhereQueue() -> std::optional<std::pair<Fiber*, FiberSource>>;
	auto tryStealFiberFrom(workerid_t victim) -> std::optional<std::pair<Fiber*, FiberSource>>;

 protected:
	WsQueue<QUEUE_SIZE>** queues;
	static thread_local WsQueue<QUEUE_SIZE> queue;

	void scheduleViaWorkStealing(Fiber& fiber);

	auto maybeRecycle(Fiber* fiber) -> bool;

	auto nextFiberViaWorkStealing() -> std::pair<Fiber*, FiberSource>;

	auto nextFiberResultViaWorkStealing() -> NextFiberResult;

 public:
	AbstractWorkStealingScheduler(Runtime& runtime, RuntimeStrategy& strategy);

	friend class Runtime;
};
