// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#include "LawsStrategy.hpp"

#include <functional>

#include "strategies/laws/LawsDispatcher.hpp"			// for LawsDispatcher
#include "strategies/laws/LawsScheduler.hpp"			// for LawsScheduler
#include "strategies/laws/LawsStrategyStats.hpp"	// for LawsStrategyStats
#include "strategies/laws/LawsWorkerStats.hpp"

thread_local LawsWorkerStats LawsStrategy::stats;

LawsStrategy::LawsStrategy(Runtime& runtime)
		: AbstractWorkStealingStrategy(runtime),
			scheduler(runtime, *this),
			dispatcher(runtime),
			allWorkerStats([] { return &stats; }, runtime) {}

auto LawsStrategy::getScheduler() -> LawsScheduler& { return scheduler; }

auto LawsStrategy::getDispatcher() -> LawsDispatcher& { return dispatcher; }

auto LawsStrategy::getStats() -> std::shared_ptr<RuntimeStrategyStats> {
	return std::make_shared<LawsStrategyStats>(*this);
}
