// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#pragma once

#include <cstdint>
#include <memory>

#include "WorkerLocalData.hpp"
#include "strategies/AbstractWorkStealingScheduler.hpp"
#include "strategies/AbstractWorkStealingStrategy.hpp"
#include "strategies/laws/LawsDispatcher.hpp"
#include "strategies/laws/LawsScheduler.hpp"

class LawsStrategyFactory;
class LawsStrategyStats;
class Runtime;
class RuntimeStrategyStats;
struct LawsWorkerStats;

class LawsStrategy : public AbstractWorkStealingStrategy {
 private:
	enum class FiberSource : uintptr_t {
		local = static_cast<uintptr_t>(AbstractWorkStealingScheduler::FiberSource::local),
		hintWsq = static_cast<uintptr_t>(AbstractWorkStealingScheduler::FiberSource::hintWsq),
		hintAq = static_cast<uintptr_t>(AbstractWorkStealingScheduler::FiberSource::hintAq),
		stolen = static_cast<uintptr_t>(AbstractWorkStealingScheduler::FiberSource::stolen),
		anywhereQueue =
				static_cast<uintptr_t>(AbstractWorkStealingScheduler::FiberSource::anywhereQueue),
		fromPriority,
	};

	LawsScheduler scheduler;
	LawsDispatcher dispatcher;

	static thread_local LawsWorkerStats stats;

	WorkerLocalData<LawsWorkerStats> allWorkerStats;

	LawsStrategy(Runtime& runtime);

	auto getScheduler() -> LawsScheduler& override;

	auto getDispatcher() -> LawsDispatcher& override;

 public:
	auto getStats() -> std::shared_ptr<RuntimeStrategyStats> override;

	friend LawsScheduler;
	friend LawsDispatcher;
	friend LawsStrategyStats;
	friend LawsStrategyFactory;
};
