// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Fischer
#include <unistd.h>	 // for pipe

#include <cassert>	// for assert
#include <cstdint>	// for uint64_t, int32_t
#include <cstring>	// for memcmp

#include "Common.hpp"			// for DIE_MSG_ERRNO, DIE_MSG
#include "io/Future.hpp"	// for ReadFuture, CloseFuture, WriteFuture

using emper::io::CloseFuture;
using emper::io::ReadFuture;
using emper::io::WriteFuture;

static const size_t MEMSIZE = 10 << 20;

void emperTest() {
	auto* memOut = new char[MEMSIZE];
	auto* memIn = new char[MEMSIZE];
	// NOLINTNEXTLINE(modernize-avoid-c-arrays)
	int pipefds[2];

	if (pipe(pipefds) == -1) {
		DIE_MSG_ERRNO("pipe failed");
	}

	WriteFuture writeFuture(pipefds[1], memOut, MEMSIZE, 0, true);
	writeFuture.submit();

	ReadFuture readFuture(pipefds[0], memIn, MEMSIZE, 0, true);
	readFuture.submit();

	int32_t res = writeFuture.wait();
	assert(res == MEMSIZE);

	res = readFuture.wait();
	assert(res == MEMSIZE);

	assert(memcmp(memOut, memIn, MEMSIZE) == 0);

	CloseFuture closeWriteSide(pipefds[1]);
	assert(closeWriteSide.submitAndWait() == 0);

	CloseFuture closeReadSide(pipefds[0]);
	assert(closeReadSide.submitAndWait() == 0);

	delete[] memOut;
	delete[] memIn;
}
